from lenstronomy.LensModel.lens_model import LensModel
import numpy.testing as npt
import numpy as np
from pyHalo.utilities import interpolate_ray_paths, de_broglie_wavelength, delta_sigma, ITSampling, \
    inverse_transform_sampling, inverse_transform_sampling_from_cdf
from pyHalo.Cosmology.cosmology import Cosmology
import pytest

class TestUtilities(object):

    def test_interp_ray_paths(self):

        cosmo = Cosmology()
        x = [1.4, -1.]
        y = [-0.4, 0.2]
        lens_model = LensModel(['SIS'], z_source=2., lens_redshift_list=[0.5], multi_plane=True)
        kwargs_lens = [{'theta_E': 1., 'center_x': 0., 'center_y': 0.}]

        interpx, interpy = interpolate_ray_paths(x, y, lens_model, kwargs_lens, 2.,
                                                             terminate_at_source=False, source_x=None, source_y=None,
                                                             evaluate_at_mean=False)
        zarray = np.linspace(0., 2., 100)
        dc = [cosmo.D_C_transverse(zi) for zi in zarray]
        interpx_mean, interpy_mean = interpolate_ray_paths(x, y, lens_model, kwargs_lens, 2.,
                                                                       terminate_at_source=False, source_x=None,
                                                                       source_y=None,
                                                                       evaluate_at_mean=True, cosmo=cosmo)
        for dci in dc:
            x, y = 0.5 * (interpx[0](dci) + interpx[1](dci)), 0.5 * (interpy[0](dci) + interpy[1](dci))
            npt.assert_almost_equal(x, interpx_mean[0](dci))
            npt.assert_almost_equal(y, interpy_mean[0](dci))

        x = [1.4, -1.]
        y = [-0.4, 0.2]
        source_x, source_y = 0.2, 0.5
        interpx, interpy = interpolate_ray_paths(x, y, lens_model, kwargs_lens, 2.,
                                                             terminate_at_source=True, source_x=source_x,
                                                             source_y=source_y,
                                                             evaluate_at_mean=True)
        npt.assert_almost_equal(interpx[0](dc[-1]), source_x)
        npt.assert_almost_equal(interpy[0](dc[-1]), source_y)

    def test_uldm_functions(self):

        log10_m_uldm=-22 # log(eV)
        v=200 # km/s
        z_lens,z_source=0.5,1.5
        m=1e13 #M_solar
        rein=6. #kpc

        lambda_dB = de_broglie_wavelength(log10_m_uldm,v)
        npt.assert_almost_equal(lambda_dB,0.6)
        delta_kappa = delta_sigma(m,z_lens,rein,lambda_dB)
        npt.assert_almost_equal(delta_kappa/80837585.696, 1, 2)

    def test_inverse_transform_sampling(self):

        mu = 2.0
        sigma = 1.5
        func = lambda x: np.exp(-0.5 * (x - mu) ** 2 / sigma**2)
        x = np.linspace(mu - 5*sigma, mu + 5*sigma, 1000)
        x_samples = inverse_transform_sampling(x, func, (), 100000)
        npt.assert_almost_equal(np.mean(x_samples)/mu, 1.0, 2)
        npt.assert_almost_equal(np.std(x_samples)/sigma, 1.0, 2)

    def test_ITSampling(self):

        mu = 2.14
        sigma = 0.35
        x = np.random.normal(mu, sigma, 250000)
        sampler = ITSampling.from_samples(x)
        x_samples = sampler(100000)
        npt.assert_almost_equal(np.mean(x_samples)/mu, 1.0, 2)
        npt.assert_almost_equal(np.std(x_samples)/sigma, 1.0, 2)


if __name__ == '__main__':

    pytest.main()



import pytest
import numpy as np
import numpy.testing as npt
from pyHalo.Rendering.MassFunctions.mass_function_base import CDMPowerLaw, WDMPowerLaw, MixedWDMPowerLaw


class TestCDMPowerLaw(object):

    def setup_method(self):

        self.log_mlow = 5.7
        self.log_mhigh = 8.7
        self.power_law_index = -1.9
        self.draw_poisson = False
        self.normalization = 1e11
        self.mass_function = CDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                self.draw_poisson, self.normalization)

    def test_zeroth_moment(self):
        n_expected = self.mass_function.n_mean
        n_rendered = len(self.mass_function.draw())
        npt.assert_almost_equal(n_rendered / n_expected, 1, 3)

    def test_first_moment(self):
        expected = self.mass_function.first_moment
        rendered = np.sum(self.mass_function.draw())
        npt.assert_almost_equal(expected / rendered, 1, 2)

class TestWDMPowerLaw(object):

    def setup_method(self):

        self.log_mlow = 5.7
        self.log_mhigh = 8.7
        self.power_law_index = -1.9
        self.draw_poisson = False
        self.normalization = 2e11
        self.log_mc = 7.
        self.a_wdm = 0.5
        self.b_wdm = 1.2
        self.c_wdm = -2.1
        self.mass_function = WDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                self.draw_poisson, self.normalization, self.log_mc, self.a_wdm, self.b_wdm, self.c_wdm)
    def test_zeroth_moment(self):
        n_expected = self.mass_function.n_mean
        n_rendered = len(self.mass_function.draw())
        npt.assert_almost_equal(n_rendered / n_expected, 1, 2)

    def test_first_moment(self):
        expected = self.mass_function.first_moment
        rendered = np.sum(self.mass_function.draw())
        npt.assert_almost_equal(expected / rendered, 1, 2)

class TestMixedWDMPowerLaw(object):

    def setup_method(self):

        self.log_mlow = 5.7
        self.log_mhigh = 8.7
        self.power_law_index = -1.9
        self.draw_poisson = False
        self.normalization = 2e11
        self.log_mc = 7.
        self.a_wdm = 0.5
        self.b_wdm = 1.2
        self.c_wdm = -1.6
        self.f_mixed = 0.6
        self.mass_function = MixedWDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                self.draw_poisson, self.normalization, self.log_mc, self.a_wdm, self.b_wdm,
                                         self.c_wdm, self.f_mixed)

        self.mass_function_mixed_pure_wdm = MixedWDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                         self.draw_poisson, self.normalization, self.log_mc, self.a_wdm, self.b_wdm,
                                         self.c_wdm, 0.0)

        self.mass_function_mixed_pure_cdm = MixedWDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                                            self.draw_poisson, self.normalization, self.log_mc,
                                                            self.a_wdm, self.b_wdm,
                                                            self.c_wdm, 1.0)

        self.mass_function_pure_wdm = WDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                                        self.draw_poisson, self.normalization, self.log_mc, self.a_wdm,
                                                        self.b_wdm,
                                                        self.c_wdm)

        self.mass_function_pure_cdm = CDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
                                                  self.draw_poisson, self.normalization)

    def test_zeroth_moment(self):
        n_expected = self.mass_function.n_mean
        n_rendered = len(self.mass_function.draw())
        npt.assert_almost_equal(n_rendered / n_expected, 1, 2)

    def test_first_moment(self):
        expected = self.mass_function.first_moment
        rendered = np.sum(self.mass_function.draw())
        npt.assert_almost_equal(expected / rendered, 1, 2)

    def test_pure_wdm(self):

        first_moment_wdm = self.mass_function_mixed_pure_wdm.first_moment
        first_moment_mixed_pure_wdm = self.mass_function_pure_wdm.first_moment
        npt.assert_almost_equal(first_moment_mixed_pure_wdm/first_moment_wdm, 1.0, 8)

    def test_pure_cdm(self):

        first_moment_cdm = self.mass_function_mixed_pure_cdm.first_moment
        first_moment_mixed_pure_cdm = self.mass_function_pure_cdm.first_moment
        npt.assert_almost_equal(first_moment_cdm/first_moment_mixed_pure_cdm, 1.0, 8)

# class TestTabulated(object):
#
#     def setup_method(self):
#
#         self.log_mlow = 6.0
#         self.log_mhigh = 8.0
#         self.power_law_index = -1.9
#         self.draw_poisson = False
#         self.normalization = 1e10
#         self.mass_function_powerlaw = CDMPowerLaw(self.log_mlow, self.log_mhigh, self.power_law_index,
#                                          self.draw_poisson, self.normalization)
#
#         m = np.logspace(6, 8, 1000)
#         dndm = self.normalization * m ** self.power_law_index
#         self.mass_function_tabulated = Tabulated(self.log_mlow, self.log_mhigh, self.draw_poisson,
#                                                  m, dndm)
#
#     def test_zeroth_moment(self):
#
#         npt.assert_almost_equal(self.mass_function_powerlaw.n_mean/self.mass_function_tabulated.n_mean, 1.0, 3)
#
#     def test_first_moment(self):
#
#         npt.assert_almost_equal(self.mass_function_powerlaw.first_moment/self.mass_function_tabulated.first_moment, 1.0, 3)
#         rendered = np.sum(self.mass_function_tabulated.draw())
#         expected = np.sum(self.mass_function_powerlaw.draw())
#         npt.assert_almost_equal(rendered/expected,1.0)


if __name__ == '__main__':
   pytest.main()



import numpy as np
import numpy.testing as npt
import pytest
from pyHalo.Rendering.SpatialDistributions.nfw import ProjectedNFW, CNFW
from lenstronomy.Util.analysis_util import azimuthalAverage
import matplotlib.pyplot as plt


class TestProjectedNFW(object):

    def setup_method(self):

        rmax2d_arcsec = 10.0
        rs_kpc = 60
        r200_kpc = 550

        self.arcsec_to_kpc = 6.2
        self.rs_arcsec = rs_kpc / self.arcsec_to_kpc
        self.rmax2d_arcsec = rmax2d_arcsec
        self.r200_arcsec = r200_kpc / self.arcsec_to_kpc
        self.rcore_arcsec = 0.5 * self.rs_arcsec
        self.nfw = ProjectedNFW(self.rmax2d_arcsec, self.rs_arcsec, self.rcore_arcsec,
                                self.r200_arcsec, self.arcsec_to_kpc)

    def test_limit(self):

        x, y, r3 = self.nfw.draw(50000)
        r2_kpc = np.hypot(x, y)
        rmax_kpc = self.rmax2d_arcsec * self.arcsec_to_kpc
        npt.assert_almost_equal(max(r2_kpc)/rmax_kpc, 1, 2)
        rmax_3d_kpc = self.r200_arcsec * self.arcsec_to_kpc
        npt.assert_almost_equal(max(r3)/rmax_3d_kpc,1,1)

    def test_profile(self):

        rbins = np.linspace(0.005*self.rmax2d_arcsec, self.rmax2d_arcsec, 100)

        x_kpc, y_kpc, r3d_kpc = self.nfw.draw(3000000)
        x_arcsec, y_arcsec = x_kpc / self.arcsec_to_kpc, y_kpc / self.arcsec_to_kpc
        r_arcsec = np.hypot(x_arcsec, y_arcsec)
        number_density = []
        for i in range(0, len(rbins)-1):
            cond1 = r_arcsec >= rbins[i]
            cond2 = r_arcsec < rbins[i+1]
            cond = np.logical_and(cond1, cond2)
            area = rbins[i+1]**2 - rbins[i] ** 2
            number_density.append(np.sum(cond)/area)
        number_density = np.array(number_density)

        #plt.loglog(rbins[0:-1], number_density, color='k')

        cnfw_profile = CNFW()
        number_density_true = cnfw_profile.density_2d(rbins, 0.0, self.rs_arcsec, 1.0, self.rcore_arcsec)
        rescale = np.mean(number_density[0:10])/number_density_true[0]
        number_density_true *= rescale
        ratio = []
        for i in range(0, len(rbins[0:-1])):
            ratio.append(number_density_true[i]/number_density[i])
        ratio = np.array(ratio)
        npt.assert_array_less(abs(1-np.sum(ratio)/len(ratio)), 0.05)
        # plt.loglog(rbins, number_density_true, color='b')
        # plt.gca().axvline(self.rcore_arcsec)
        # plt.xlim(0.01, 20)
        # plt.show()

if __name__ == '__main__':
     pytest.main()
